import os
from flask import current_app
from flask_migrate import Migrate
from flask_sqlalchemy import SQLAlchemy
from sqlalchemy import MetaData

# https://alembic.sqlalchemy.org/en/latest/naming.html
metadata = MetaData(
    naming_convention={
        "pk": "pk_%(table_name)s",
        "ix": "ix_%(table_name)s_%(column_0_name)s",
        "uq": "uq_%(table_name)s_%(column_0_name)s",
        "ck": "ck_%(table_name)s_%(constraint_name)s",
        "fk": "fk_%(table_name)s_%(column_0_name)s_%(referred_table_name)s",
    }
)


db = SQLAlchemy(metadata=metadata)
migrate = Migrate()


@migrate.configure
def configure_alembic(config):
    # Load migration paths from plugins
    migrations = [str(p.migrations_path) for p in current_app.config["FG_PLUGINS"].values() if p and p.migrations_path]
    if len(migrations) > 0:
        # Get configured paths
        paths = config.get_main_option("version_locations")
        # Get configured path seperator
        sep = config.get_main_option("version_path_separator", "os")
        if paths:
            # Insert configured paths at the front, before plugin migrations
            migrations.insert(0, config.get_main_option("version_locations"))
        sep = os.pathsep if sep == "os" else " " if sep == "space" else sep
        # write back seperator (we changed it if neither seperator nor locations were specified)
        config.set_main_option("version_path_separator", sep)
        config.set_main_option("version_locations", sep.join(migrations))
        print(config.get_main_option("version_locations"))
    return config


def case_sensitive(s):
    """
        Compare string as case sensitive on the database

    Args:
        s: string to compare

    Example:
        User.query.filter(User.name == case_sensitive(some_string))
    """
    if db.session.bind.dialect.name == "mysql":
        from sqlalchemy import func

        return func.binary(s)
    return s
